package francois.pessaux.androjig.android;

import java.io.File;
import java.util.Random ;

import francois.pessaux.android.R;
import francois.pessaux.androjig.android.GameStruct.Game ;

import android.app.Application ;
import android.content.Context;
import android.media.AudioManager;
import android.media.SoundPool;
import android.os.Environment;


public class AndroJigApp extends Application {
  /** Random generator used everywhere we need random integers. */
  public static Random randomGenerator = new Random () ;
  /** Global sound manager. Used everywhere a sound must be played. */
  public static SoundPool soundPool ;
  /** Volume at which sounds must be played. */
  public static float soundVolume ;
  /** Width of the device's display, i.e. with vertical decorations (if there
      are some). */
  public static int deviceDisplayWidth ;
  /** Width of the device's display, i.e. with horizontal decorations (status
      bar, tool bar, menu bar, etc. */
  public static int deviceDisplayHeight ;
  /** Current global running game. We always have at most 1 running game. */
  public static Game currentGame = null ;
  
  /** Identifier of the sound played when pieces collapse. */
  public static int clipSoundId ;
  /** Identifier of the sound played when the game gets won. */
  public static int youpiSoundId ;

  /** Identifier of the Activity handling image selection. */
  public static final int ACTIVITY_SELECT_PICTURE = 1 ;
  /** Identifier of the Activity handling game setting. */
  public static final int ACTIVITY_SETUP_GAME = 2 ;
  /** Identifier of the Activity handling the play of the game. */
  public static final int ACTIVITY_PLAY_GAME = 3 ;
  /** Identifier of the Activity handling the play of the game. */
  public static final int ACTIVITY_PICK_WEB_MAGE = 4 ;

  
  /** Name of the directory located on the sdcadr where we save persistent
      data. */
  private static final String gameDirectoryName = "AndrojigData" ;
  /** Name of the savegame file. */
  public static final String saveGameFileName =
  	gameDirectoryName + File.separatorChar + "aj_save" ;
  public static final String imgDownloadTmpFileName =
  	gameDirectoryName + File.separatorChar + "aj_tmpd" ;
 
  
  
  private static AndroJigApp instance;

  public AndroJigApp ()
  {
  	instance = this ;
  }

  
  
  public static Context getContext ()
  {
    return (instance) ;
  }
  
  
  
  /** Here initialization code performed once for all when application is
      created. */
  @Override
  public void onCreate ()
  {
    super.onCreate () ;
    PieceStruct.initConstants () ;
		/* First parameter defines the number of channels which should be played
		   in parallel, last one currently not used. */
    soundPool = new SoundPool (1, AudioManager.STREAM_MUSIC, 0) ;
    clipSoundId = soundPool.load (this, R.raw.clipsound, 1) ;
    youpiSoundId = soundPool.load (this, R.raw.youpi, 1) ;
    /* Get the user sound settings. */
		AudioManager audioManager =
			(AudioManager) getSystemService (AUDIO_SERVICE) ;
		float actualVolume =
		  (float) audioManager.getStreamVolume (AudioManager.STREAM_MUSIC) ;
		float maxVolume =
			(float) audioManager.getStreamMaxVolume (AudioManager.STREAM_MUSIC) ;
		soundVolume = actualVolume / maxVolume ;
		/* Check if the data directory exists and if not, create it. */
		File sdCard = Environment.getExternalStorageDirectory () ; 
		File gameDirectory = new File (sdCard, gameDirectoryName) ;
		if (! gameDirectory.exists ()) gameDirectory.mkdir () ;
  }

}
