package francois.pessaux.androjig.android;


import java.io.File ;
import java.io.IOException ;

import francois.pessaux.android.R;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;

import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Environment ;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View ;
import android.widget.Button ;


public class AndroJigMainActivity extends Activity {
	private static final int DIALOG_ID_ERROR_LOADING_SAVEGAME = 1 ;
	
  private Button buttonContinueGame ;
  
  
  
  /** Called when the activity is first created. */
  @Override
  public void onCreate (Bundle savedInstanceState)
  {
    super.onCreate (savedInstanceState) ;
    setContentView (R.layout.main) ;
    buttonContinueGame = (Button) findViewById (R.id.buttonContinueGame) ;
    
    /* Check if there is a save game. If so, enable the button to continue this
       saved game. Attention, this is done here in addition to in [onResume]
       because when the program just start, [onResume] seems to be called before
       the view is created, hence with [buttonContinueGame] still null, so it
       it not possible to perform the initial update of the button state
       directly in [onResume]. */
    File sdCard = Environment.getExternalStorageDirectory () ; 
    File file = new File (sdCard, AndroJigApp.saveGameFileName) ;
    buttonContinueGame.setEnabled (file.exists ()) ;
  }
  
  
  
  /** Called each time the activity comes to the foreground. */
  @Override
  public void onResume ()
  {
  	super.onResume () ;
    /* Check if there is a save game. If so, enable the button to continue this
       saved game. Attention, at the initial step in this method, the view
       seems to be not yet created, hence [buttonContinueGame] is null.
       In this case, we don't attempt to change its state and instead, this
       will be done in [onCreate]. After initial step here, i.e. each time we
       arrive here because the Activity we launched ended and gave us back the
       hand, [buttonContinueGame] won't be null and we will be able to change
       its state. */
    if (buttonContinueGame != null) {
      File sdCard = Environment.getExternalStorageDirectory () ; 
      File file = new File (sdCard, AndroJigApp.saveGameFileName) ;
      buttonContinueGame.setEnabled (file.exists ()) ;
    }
  }
  
  
  
  @Override
  public boolean onCreateOptionsMenu (Menu menu)
  {
    MenuInflater inflater = getMenuInflater () ;
    inflater.inflate (R.menu.mainmenu, menu) ;
    return (true) ;
  }

  
  /** Handler called when a menu item is selected. */
  @Override
  public boolean onOptionsItemSelected (MenuItem item)
  {
    switch (item.getItemId ()) {
    case R.id.itemAbout:
    	Dialog dialog = new Dialog (this) ;
    	dialog.setContentView (R.layout.about) ;
    	dialog.setTitle (R.string.aboutMenuStr) ;
    	dialog.show () ;
    	/* Return [true] to tell we handled the event. */
    	return (true) ;
      // break ;
    	
    case R.id.itemHelp:
    	/* Open the help activity from which we do not expect any result. */
    	Intent intent = new Intent (this, HelpActivity.class) ;
      startActivity (intent) ;
    	return (true) ;
      // break ;

    case R.id.itemQuit:
    	System.exit (0) ;
    	// break ;
    	
    default:
      /* In all other cases, let the system handling the event and return the
         value the system returned. */
      return (super.onOptionsItemSelected (item)) ;
      // break ;
    }
  }

  
  
  @Override
  protected Dialog onCreateDialog (int id)
  {  	
    AlertDialog.Builder builder ;
    AlertDialog alert ;
    builder = new AlertDialog.Builder (this) ;
    switch (id) {
      
    case DIALOG_ID_ERROR_LOADING_SAVEGAME:
      builder.setMessage
        (R.string.errLoadingSavegameStr)
        .setCancelable (false)
        .setPositiveButton
          (R.string.dismissStr, new DialogInterface.OnClickListener () {
          	public void onClick (DialogInterface dialog, int id) { }}) ;
    break ;
    
    default:
      builder.setMessage (R.string.unknownMsgIdStr)
      .setCancelable (false)
      .setPositiveButton
      	(R.string.dismissStr, new DialogInterface.OnClickListener () {
      		public void onClick (DialogInterface dialog, int id) { }}) ;
      break ;
    }
    
    alert = builder.create () ;
    alert.show () ;
    return (super.onCreateDialog (id)) ;
  }

  
  
  /** Handler called then the button to start a game parameters setting is
      pressed. */
  public void onClickSetupGame (View view)
  {
  	/* Very dirty hack to retrieve the real size of the display area,
  	   without status and menu bars. In fact, we use the fact that the play
  	   area has only one view that extends on the whole available surface,
  	   like this is the case for the initial screen view. So, knowing the
  	   initial screen's view size, we know the play area view. We put this
  	   code here because, we are forced to go here to start a game and at this
  	   point, we are sure that the initial screen's view is display and has
  	   it right dimensions. */
  	View mainViewRootLayout = (View) findViewById (R.id.layoutrootmain) ;
    AndroJigApp.deviceDisplayWidth = mainViewRootLayout.getWidth () ;
    AndroJigApp.deviceDisplayHeight = mainViewRootLayout.getHeight () ;
    
    Intent intent = new Intent (this, SetupGameActivity.class) ;
    startActivityForResult (intent, AndroJigApp.ACTIVITY_SETUP_GAME) ;
  }
  
  
  
  /** Handler called then the button to continue a saved game is pressed.
      It assumes that the button was enabled only if a savegame was really
      found. */
  public void onClickContinueGame (View view)
  {
  	/* See comment in [onClickSetupGame]. */
  	View mainViewRootLayout = (View) findViewById (R.id.layoutrootmain) ;
    AndroJigApp.deviceDisplayWidth = mainViewRootLayout.getWidth () ;
    AndroJigApp.deviceDisplayHeight = mainViewRootLayout.getHeight () ;
    
    try {
    	AndroJigApp.currentGame =
    		GameLoad.loadGame (AndroJigApp.saveGameFileName) ;
    	/* Game is loaded, then discard the savegame file. */
    	GameLoad.deleteSavegameFile (AndroJigApp.saveGameFileName) ;
    	/* Set width and height of the play area view here because it is not
    	   done in the savegame loading routine but is need to have motion of
    	   pieces working. When starting a new game this is done by the setup
    	   Activity in method [startNewGame]. */
    	PlayAreaView.playAreaWidth = AndroJigApp.deviceDisplayWidth ;
    	PlayAreaView.playAreaHeight = AndroJigApp.deviceDisplayHeight ;
      if (AndroJigApp.currentGame != null) {
      	Intent intent = new Intent (this, PlayGameActivity.class) ;
        startActivity (intent) ;
      }
    }
    catch (IOException e) {
    	this.showDialog (DIALOG_ID_ERROR_LOADING_SAVEGAME) ;
    }
  }
  
  
  /** Method called when an Activity launched by the present one ends. */
  protected void onActivityResult (int requestCode, int resultCode, Intent data)
  {
    switch (requestCode) {
    case AndroJigApp.ACTIVITY_SETUP_GAME:
      if ((resultCode == RESULT_OK) && (AndroJigApp.currentGame != null)) {
        Intent intent = new Intent (this, PlayGameActivity.class) ;
        startActivity (intent) ;
      }
      break ;
    }
  }

}
