package francois.pessaux.androjig.android;

import android.graphics.Bitmap ;
import francois.pessaux.androjig.android.PieceStruct.Edge ;
import francois.pessaux.androjig.android.PieceStruct.Piece ;
import francois.pessaux.androjig.android.PieceStruct.Pieces_list ;

public class GameStruct {

  private static int TAGTOLONG (byte a, byte b,byte c,byte d) 
  {
    return ((int)(((a)<<24)|((b)<<16)|((c)<<8)|(d))) ;
  }
  
  public static int SAVEMAGICNUMBER =
    TAGTOLONG ((byte) 'A',(byte) 'J',(byte) 'I', (byte) 'G') ;

  
	
  /** Describes each cell of the layout, organized as a bi-dimensional
      array of such cells. Each cell record the pieces whose bounding
      box span on and a boolean flag telling if the pieces of this cell
      must be redrawn. */
  public static class Layout_cell {
    Pieces_list hosted_pieces ;
  };



  public static class Game {
  	/** Boolean telling if the original image was rotated to maximize display
  	    area usage. */
  	boolean rotated_image ;
    /** Boolean telling if the game is still living because not yet won. */
    boolean game_won ;
    /** Image width in pixels. The image width is the one really used in the
        game, i.e. possibly smaller than the original image if this latter was
        cropped to get an integer width of pieces according to the number of
        pieces requested horizontally by the user. */
    int image_width ;
    /** Image height in pixels. The image height is the one really used in the
        game, i.e. possibly smaller than the original image if this latter was
        cropped to get an integer height of pieces according to the number of
        pieces requested vertically by the user. */
    int image_height ;
    /** The scaling factor to apply to the original image. If negative of null,
        then no scaling is to apply to the image and this latter keeps it
        original size. The above image size already takes into account the
        scaling. This factor is only needed when rebuilding the image to map
        from the original picture file in case of loading a saved game. */
    float original_image_scale_factor ;
    /** Puzzle piece base width (i.e. without deformation) in pixels. */
    int piece_width ;
    /** Puzzle piece base height (i.e. without deformation) in pixels. */
    int piece_height ;
    /** Computed to be image_width / piece_width. Represents the size
        of the layout. But we could have a layout larger then image. */
    int layout_nb_columns ;
    /** Computed to be image_height / piece_height. Represents the size
        of the layout. But we could have a layout larger then image. */
    int layout_nb_lines ;
    /** Grid telling which pieces are in which lines / columns.
        We allocate it as a global array and access it by
        (y * layout_nb_columns) + x.
        Since there can be several pieces overlapping a cell, each cell contains a
        list of pieces. Pieces are set in the layout cells based on their
        bounding box. */
    Layout_cell[] layout ;
    /** Number of pieces in the game. */
    int nb_pieces ;
    /* Array of pieces, assumed to be indexed by the pieces ids. I.e. at index
       i, we find the piece whose index is i. */
    Piece[] pieces ;
    /** Head of the double-linked sorted list of edges by increasing x. */
    Edge x_sorted_edges_head ;
    /** Tail of the double-linked sorted list of edges by increasing x. */
    Edge x_sorted_edges_tail ;
    /** Head of the double-linked sorted list of edges by increasing y. */
    Edge y_sorted_edges_head ;
    /** Tail of the double-linked sorted list of edges by increasing y. */
    Edge y_sorted_edges_tail ;
    /* Reference to the graphic object representing the complete image. */
    Bitmap game_image ;
    /* Size in bytes of the data representing the image file. */
    int image_file_size ;
    /** Save of the content of the image file. Allows to prevent problems in case
        of image disparition between save / load game. */
    byte[] image_file_data ;
    
    
    
    public Game ()
    {
      game_won = false ;
      image_width = 0;
      image_height = 0 ;
      original_image_scale_factor = (float) 0.0 ;
      piece_width = 0 ;
      piece_height = 0;
      layout_nb_columns = 0 ;
      layout_nb_lines = 0 ;
      layout = null ;
      nb_pieces = 0 ;
      pieces = null ;
      x_sorted_edges_head = null ;
      x_sorted_edges_tail = null ;
      y_sorted_edges_head = null ;
      y_sorted_edges_tail = null ;
      game_image = null ;
      image_file_size = 0 ;
      image_file_data = null ;
    }
  }

}
