package francois.pessaux.androjig.android;

import francois.pessaux.android.R;
import francois.pessaux.androjig.android.GameStruct.Game ;
import francois.pessaux.androjig.android.PieceStruct.Piece ;
import android.content.Context ;
import android.graphics.Canvas ;
import android.graphics.Paint ;
import android.view.MotionEvent ;
import android.view.View ;
import android.widget.Toast;

public class PlayAreaView extends View {
  private Paint defaultPaint ;
  private Piece holdPiece = null ;
  public static int playAreaWidth = 0 ;
  public static int playAreaHeight = 0 ;
  public static int bgRedColor = 180 ;
  public static int bgGreenColor = 180 ;
  public static int bgBlueColor = 180 ;
  
  
  
  public PlayAreaView (Context context)
  {
    super (context) ;
    setFocusable (true) ;
    defaultPaint = new Paint () ;
  }



  public boolean onTouchEvent (MotionEvent event)
  {
    Game currentGame = AndroJigApp.currentGame ;

    switch (event.getAction ()) {
    case MotionEvent.ACTION_DOWN:
      if (currentGame != null) {
        int x = (int) event.getX () ;
        int y = (int) event.getY () ;
        holdPiece = GameManips.findClosestPiece (currentGame, x, y) ;
      }
      break ;
        
    case MotionEvent.ACTION_MOVE:
      if (currentGame != null) {
        if (holdPiece != null) {
          int x = (int) event.getX () ;
          int y = (int) event.getY () ;
          /* Ensure that it is impossible to move the hold piece beyond the
             limits of the view. This way, we are always sure that the user will
             be able to pick it later. */
          if ((x >= 0) && (x < playAreaWidth) &&
              (y >= 0) && (y < playAreaHeight)) {
            GameManips.movePieceTo (x, y, currentGame, holdPiece) ;
            invalidate () ;
          }
        }
      }
      break ;
      
    case MotionEvent.ACTION_UP:
      if ((currentGame != null) && (holdPiece != null)) {
        if (GameManips.checkCoalescence (currentGame, holdPiece)) {
        	/* Play a sound if some pieces clipped together. */
        	AndroJigApp.soundPool.play
        		(AndroJigApp.clipSoundId, AndroJigApp.soundVolume,
        		 AndroJigApp.soundVolume, 1, 0, 1f) ;
        }
        holdPiece = null ;
        invalidate () ;
        if (GameManips.isGameJustWon (currentGame)) {
          /* Play a sound if some pieces clipped together. */
          AndroJigApp.soundPool.play
            (AndroJigApp.youpiSoundId, AndroJigApp.soundVolume,
             AndroJigApp.soundVolume, 1, 0, 1f) ;
          /* TODO Disable the menu items etc. */

          /* Show a little congratulation message. */
          Toast.makeText
          	(AndroJigApp.getContext (), R.string.jigsawCompletedStr,
          	 Toast.LENGTH_LONG).show () ;
        }
      }
      break ;
      
      default:
        return (false) ;   /* Tell we didn't handle the event. */
        // break ;
    }
    return (true) ; /* Tell we handled the event. */
  }
  
  
  
  public void onDraw (Canvas canvas)
  {
	Game game = AndroJigApp.currentGame ;
	
    if (game == null) return ;
     
    canvas.drawRGB (bgRedColor, bgBlueColor, bgGreenColor) ;
    for (int i = 0; i < game.nb_pieces; i++) {
      Piece curr_piece = game.pieces[i] ;
      float top =
        (float) (curr_piece.center.x + curr_piece.delta_center_to_top_left.x) ;
      float left =
        (float) (curr_piece.center.y + curr_piece.delta_center_to_top_left.y) ;
      canvas.drawBitmap (curr_piece.piece_image, top, left, defaultPaint) ;
    }
  }
}
