package francois.pessaux.androjig.android;
import java.io.IOException ;

import francois.pessaux.android.R;
import android.app.Activity;
import android.app.AlertDialog ;
import android.app.Dialog ;
import android.content.DialogInterface ;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.FrameLayout;
import android.widget.Toast;


public class PlayGameActivity extends Activity {
	private static final int DIALOG_ID_ERROR_SAVNG_SAVEGAME = 1 ;
	
  private PlayAreaView playAreaView ;
  
  
	/** Called when the activity is first created. */
	@Override
	public void onCreate (Bundle savedInstanceState)
	{
		super.onCreate (savedInstanceState) ;
		setContentView (R.layout.game) ;
		FrameLayout frameLayout =
			(FrameLayout) findViewById (R.id.frameLayoutPlayField) ;
		playAreaView = new PlayAreaView (this) ;
		frameLayout.addView (playAreaView, 0) ;
		playAreaView.requestFocus () ;
	}


	
  @Override
  public boolean onCreateOptionsMenu (Menu menu)
  {
    MenuInflater inflater = getMenuInflater () ;
    inflater.inflate (R.menu.playmenu, menu) ;
    return (true) ;
  }

  

  private void saveAndLeaveJigsaw ()
  {
  	/* If the puzzle is not yet ended, propose to save it before leaving. */
  	if (! AndroJigApp.currentGame.game_won) {
  		try {
  			GameSave.saveGame
  				(AndroJigApp.saveGameFileName, AndroJigApp.currentGame) ;
  			Toast.makeText
      		(this, R.string.gameSavedStr, Toast.LENGTH_SHORT).show () ;
  		}
      catch (IOException e) {
        this.showDialog (DIALOG_ID_ERROR_SAVNG_SAVEGAME) ;
      }
  	}
    /* Set all the global variables describing the state of the game to
       "empty", i.e. "there is no currently running game". */
    AndroJigApp.currentGame = null ;
    PlayAreaView.playAreaWidth = 0 ;
    PlayAreaView.playAreaHeight = 0 ;
    finish () ;
  }
  
  
  
  /** Handler called when a menu item is selected. */
  @Override
  public boolean onOptionsItemSelected (MenuItem item)
  {
    switch (item.getItemId ()) {
    case R.id.itemArrangePieces:
      GameManips.arrangePiecesOnBorders (AndroJigApp.currentGame) ;
      /* Force refresh. */
      playAreaView.invalidate () ;
      return (true) ;
      // break ;
      
    case R.id.itemChangeBgColor:
      ColorPickerDialog cpd =
        new ColorPickerDialog
          (this,
           new ColorPickerDialog.OnColorChangedListener () {
              @Override
              public void colorChanged (int color)
              {
                PlayAreaView.bgRedColor = (color & 0x00FF0000) >> 16 ;
                PlayAreaView.bgBlueColor = (color & 0x0000FF00) >> 8 ;
                PlayAreaView.bgGreenColor = color & 0x000000FF ;
                /* Force refresh. */
                playAreaView.invalidate () ;
              }},
            0xFF808080) ;
      cpd.show () ;
      return (true) ;
      // break ;
      
    case R.id.itemLeaveJigsaw:
    	saveAndLeaveJigsaw () ;
      return (true) ;
      // break ;
    
    default:
      /* In all other cases, let the system handling the event and return the
         value the system returned. */
      return (super.onOptionsItemSelected (item)) ;
      // break ;
    }
  }
  
  
  
  /** Called when a "key" is pressed. We are only interested in the "back"
      button pressed, to save the current game and go back to the introduction
      screen. */
  @Override
  public boolean onKeyDown(int keyCode, KeyEvent event)
  {
  	if (keyCode == KeyEvent.KEYCODE_BACK) {
  		saveAndLeaveJigsaw () ;
  		return (true) ;
  		}
  	
    return (super.onKeyDown (keyCode, event)) ;
  }

  
  
  @Override
  protected Dialog onCreateDialog (int id)
  {
    AlertDialog.Builder builder ;
    AlertDialog alert ;
    builder = new AlertDialog.Builder (this) ;
    switch (id) {
      
    case DIALOG_ID_ERROR_SAVNG_SAVEGAME:
      builder.setMessage (R.string.errSavingSavegameStr)
      .setCancelable (false)
      .setPositiveButton
        (R.string.dismissStr, new DialogInterface.OnClickListener () {
        	public void onClick (DialogInterface dialog, int id) { }}) ;
      break ;
      
    default:
      builder.setMessage (R.string.unknownMsgIdStr)
      .setCancelable (false)
      .setPositiveButton
      	(R.string.dismissStr, new DialogInterface.OnClickListener () {
      		public void onClick (DialogInterface dialog, int id) { }}) ;
      break ;
    }
    
    alert = builder.create () ;
    alert.show () ;
    return (super.onCreateDialog (id)) ;
  }
}
