package francois.pessaux.androjig.android;

public class ResolutionSuggestions {

  public static class Resolution {
    int nb_horizontally ;
    int nb_vertically ;
    int average_piece_size ;
  };

  
  
  private static final int MIN_PIECE_SIZE = 30 ;



  /** Computes the different configurations of vertical and horizontal number
      of pieces according to the specified image width and height so that pieces
      are square and not smaller than MIN_PIECE_SIZE pixels (both in width and
      height).
      Creates an array containing the found possible resolutions. */
   public static Resolution[] computeRecommendedResolutions (int width,
  		 		                                                   int height)
  {
    float ratio ;
    int nb_min_vert, nb_max_vert, nb_min_horiz, nb_max_horiz ;
    int i, j, nb_resolutions ;
    Resolution resolutions[] ;

    if (width > height) {
      ratio = ((float) width) / ((float) height) ;
      nb_min_vert = 2 ;
      nb_max_vert = height / MIN_PIECE_SIZE ;
      nb_min_horiz = (int) (((float) nb_min_vert) * ratio) ;
      nb_max_horiz = (int) (((float) nb_max_vert) * ratio) ;

      nb_resolutions = nb_max_vert - nb_min_vert ;
      resolutions = new Resolution[nb_resolutions] ;
      
      j = 0 ;
      for (i = nb_min_vert; i < nb_max_vert; i++) {
        resolutions[j] = new Resolution () ;
        int h = (int) (((float) i) * ratio) ;
        resolutions[j].nb_horizontally = h ;
        resolutions[j].nb_vertically = i ;
        resolutions[j].average_piece_size = height / i ;
        j++ ;
      }      /* End of for (i = nb_min_vert; i < nb_max_vert; i++). */
    }        /* End of if (width > height). */
    else {
      ratio = ((float) height) / ((float) width) ;
      nb_min_horiz = 2 ;
      nb_max_horiz = width / MIN_PIECE_SIZE ;
      nb_min_vert = (int) (((float) nb_min_horiz) * ratio) ;
      nb_max_vert = (int) (((float) nb_max_horiz) * ratio) ;
      
      nb_resolutions = nb_max_horiz - nb_min_horiz ;
      
      /* May be the image was too small to respect minimum pieces size to get
         acceptable resolutions. */
      if (nb_resolutions <= 0) return (null) ;
      
      resolutions = new Resolution[nb_resolutions] ;
      
      j = 0 ;
      for (i = nb_min_horiz; i < nb_max_horiz; i++) {
        resolutions[j] = new Resolution () ;
        int v = (int) (((float) i) * ratio) ;
        resolutions[j].nb_horizontally = i ;
        resolutions[j].nb_vertically = v ;
        resolutions[j].average_piece_size = width / i ;
        j++ ;
      }      /* End of for (i = nb_min_horiz; i < nb_max_horiz; i++). */
    }        /* End of else. */

    return (resolutions) ;
  }

}
