package francois.pessaux.androjig.android;

import francois.pessaux.android.R ;
import francois.pessaux.androjig.android.GameStruct.Game ;
import francois.pessaux.androjig.android.ResolutionSuggestions.Resolution ;
import android.app.Activity;
import android.app.AlertDialog ;
import android.app.Dialog ;
import android.content.DialogInterface ;
import android.content.Intent ;
import android.database.Cursor ;
import android.graphics.Bitmap ;
import android.graphics.BitmapFactory ;
import android.net.Uri ;
import android.os.Bundle ;
import android.provider.MediaStore ;
import android.view.View ;
import android.widget.AdapterView ;
import android.widget.AdapterView.OnItemSelectedListener ;
import android.widget.ArrayAdapter ;
import android.widget.Button ;
import android.widget.Spinner ;
import android.widget.TextView ;

public class SetupGameActivity extends Activity {
	private static final int DIALOG_ID_ERROR_CREATING_GAME = 1 ;
	
  private static TextView textViewImagePath ;
  private static TextView textViewImageWidth ;
  private static TextView textViewImageHeight ;
  private static TextView textViewScaledImageWidth ;
  private static TextView textViewScaledImageHeight ;
  private static TextView textViewImageRotated ;
  private static Button buttonSetupOk ;
  private static Spinner spinnerResolutions ;
  private static TextView textViewAvPieceSize ;
  private static String selectedImagePath = null ;
  private static float imageScale = (float) 0.0 ;
  private Resolution resolutions[] = null ;
  private static int selectedResolutionIndex = -1 ;
  private static boolean rotateImage = false ;
  
  
  
  /** Called when the activity is first created. */
  @Override
  public void onCreate (Bundle savedInstanceState)
  {
    super.onCreate (savedInstanceState) ;
    setContentView (R.layout.setup) ;
    textViewImagePath = (TextView) findViewById (R.id.textViewImagePath) ;
    textViewImageWidth = (TextView) findViewById (R.id.textViewImageWidth) ;
    textViewImageHeight = (TextView) findViewById (R.id.textViewImageHeight) ;
    textViewScaledImageWidth =
    	(TextView) findViewById (R.id.textViewScaledImageWidth) ;
    textViewScaledImageHeight =
    	(TextView) findViewById (R.id.textViewScaledImageHeight) ;
    textViewImageRotated = (TextView) findViewById (R.id.textViewImageRotated) ;
    spinnerResolutions = (Spinner) findViewById (R.id.spinnerResolutions) ;
    textViewAvPieceSize = (TextView) findViewById (R.id.textViewAvPieceSize) ;
    buttonSetupOk = (Button) findViewById (R.id.buttonSetupOk) ;
    buttonSetupOk.setEnabled (false) ;
    selectedImagePath = null ;
    selectedResolutionIndex = -1 ;
    PlayAreaView.playAreaWidth = 0 ;
    PlayAreaView.playAreaHeight = 0 ;
  }
  
    
  
  protected void onActivityResult (int requestCode, int resultCode, Intent data)
  {
  	Uri selectedImageUri ;
  	
    switch (requestCode) {
    case AndroJigApp.ACTIVITY_PICK_WEB_MAGE:
    	if ((resultCode == RESULT_OK) && (data != null)) {
    		selectedImageUri = data.getData () ;
    		if (selectedImageUri != null)
    			updateInterfaceOnImagePicked (selectedImageUri.getPath ()) ;
    	}
    	break ;
    	
    case AndroJigApp.ACTIVITY_SELECT_PICTURE:
      if ((resultCode == RESULT_OK) && (data != null)) {
        selectedImageUri = data.getData () ;
        if (selectedImageUri != null)
        	updateInterfaceOnImagePicked (getPath (selectedImageUri)) ;
      }
      break ;
    }
  }
      
  
      
  public void onClickPickImage (View view)
  {/* For real device */
    Intent intent = new Intent () ;
    intent.setType ("image/*") ;
    intent.setAction (Intent.ACTION_GET_CONTENT) ;
    startActivityForResult
      (Intent.createChooser
        (intent, getResources().getString (R.string.selectPictureStr)),
        AndroJigApp.ACTIVITY_SELECT_PICTURE) ;
    //updateInterfaceOnImagePicked ("/sdcard/test.png") ;
  }

  
  
  public void onClickSetupCancel (View view)
  {
    AndroJigApp.currentGame = null ;
    PlayAreaView.playAreaWidth = 0 ;
    PlayAreaView.playAreaHeight = 0 ;
    /* Set the return value of the activity to "false". */
    setResult (RESULT_CANCELED) ;
    finish () ;
  }
  
  
  
  public void onClickSetupOk (View view)
  {
    if ((resolutions == null) || (selectedResolutionIndex == -1)) return ;
    Resolution resolution = resolutions[selectedResolutionIndex] ;
    startNewGame
      ((selectedImagePath != null ? selectedImagePath : "/sdcard/test.png"),
       resolution.nb_horizontally, resolution.nb_vertically, (float) imageScale,
       AndroJigApp.deviceDisplayWidth, AndroJigApp.deviceDisplayHeight,
       rotateImage) ;
    /* Set the return value of the activity to "true". */
    setResult (RESULT_OK) ;
    finish () ;
  }
  
  
  
  public void onClickDontPress (View view)
  {
  	Intent intent = new Intent (this, WebImagePickActivity.class) ;
    startActivityForResult (intent, AndroJigApp.ACTIVITY_PICK_WEB_MAGE) ;
  }
  
  
  	
  @Override
  protected Dialog onCreateDialog (int id)
  {
    AlertDialog.Builder builder ;
    AlertDialog alert ;
    builder = new AlertDialog.Builder (this) ;
    switch (id) {
            
    case DIALOG_ID_ERROR_CREATING_GAME:
      builder.setMessage (R.string.errCreatingGameStr)
        .setCancelable (false)
        .setPositiveButton
        	(R.string.dismissStr, new DialogInterface.OnClickListener () {
        		public void onClick (DialogInterface dialog, int id) { }}) ;
      break ;
            
    default:
      builder.setMessage (R.string.unknownMsgIdStr)
      .setCancelable (false)
      .setPositiveButton
      	(R.string.dismissStr, new DialogInterface.OnClickListener () {
      		public void onClick (DialogInterface dialog, int id) { }}) ;
      break ;
    }
    
    alert = builder.create () ;
    alert.show () ;
    return (super.onCreateDialog (id)) ;
  }
  
  
  
  private void updateInterfaceOnImagePicked (String imagePath)
  {
  	if (imagePath == null) return ;
    Bitmap bitmap = BitmapFactory.decodeFile (imagePath) ;
    if (bitmap == null) return ;

    int raw_width = bitmap.getWidth () ;
    int raw_height = bitmap.getHeight () ;
    
    /* Force image rotation if it is taller than wide and its height is bigger
       than the device height. Hence we will maximize the used surface of the
       device display. */
    if ((raw_width < raw_height) && (raw_height > AndroJigApp.deviceDisplayHeight)) {
    	int tmp = raw_width ;
    	raw_width = raw_height ;
    	raw_height = tmp ;
    	rotateImage = true ;
    	textViewImageRotated.setText (R.string.yesStr) ;
    }
    else {
    	rotateImage = false ;
    	textViewImageRotated.setText (R.string.noStr) ;
    }
    	
    textViewImageWidth.setText (String.valueOf (raw_width)) ;
    textViewImageHeight.setText (String.valueOf (raw_height)) ;
    
    textViewImagePath.setText (imagePath) ;
    selectedImagePath = imagePath ;
        
    /* Compute scaling to get the image fitting in the display area. */
    float scale_x = (float) 1.0 ;
    if (raw_width > AndroJigApp.deviceDisplayWidth) {
    	scale_x = (float) AndroJigApp.deviceDisplayWidth / (float) raw_width ;
    }
    float scale_y = (float) 1.0 ;
    if (raw_height > AndroJigApp.deviceDisplayHeight) {
    	scale_y = (float) AndroJigApp.deviceDisplayHeight / (float) raw_height ;
    }
    imageScale = Math.min (scale_x, scale_y) ;
    
    int width = (int) ((float) raw_width * imageScale) ;
    int height = (int) ((float) raw_height * imageScale) ;
    textViewScaledImageWidth.setText (String.valueOf (width)) ;
    textViewScaledImageHeight.setText (String.valueOf (height)) ;

    
    resolutions =
      ResolutionSuggestions.computeRecommendedResolutions (width, height) ;
    int nbRes = (resolutions == null ? 0 : resolutions.length) ;
    if (nbRes == 0) {
      buttonSetupOk.setEnabled (false) ;
      resolutions = null ;
      selectedResolutionIndex = -1 ;
      return ;
    }
    
    String[] resStrArray = new String [nbRes] ;
    for (int i = 0; i < nbRes; i++) {
      Resolution res = resolutions[i] ;
      resStrArray[i] =
        (String.valueOf (res.nb_horizontally)) + "x" +
        (String.valueOf (res.nb_vertically)) ;
    }
    
    ArrayAdapter<CharSequence> adapter =
      new ArrayAdapter<CharSequence>
    		(this, android.R.layout.simple_spinner_item, resStrArray) ;
    adapter.setDropDownViewResource
    	(android.R.layout.simple_spinner_dropdown_item) ;
    spinnerResolutions.setAdapter (adapter) ;
    spinnerResolutions.setOnItemSelectedListener
      (new OnItemSelectedListener()
      {

        @Override
        public void onItemSelected (AdapterView<?> parent, View view, int pos,
                                    long id)
        {
          if (pos >= 0) {
            /* Update the variable recording the currently selection resolution
               index in the resolutions array. */
            selectedResolutionIndex = pos ;
            /* Update the 2 average pieces sizes indicators. */
            textViewAvPieceSize.setText
              (resolutions[pos].average_piece_size + "") ;
          }
        }

        @Override
        public void onNothingSelected (AdapterView<?> arg0) { }
      }) ;
    spinnerResolutions.setSelection (0) ;
    buttonSetupOk.setEnabled (true) ;
    bitmap = null ;
  }
  
  
  
  private String getPath (Uri uri)
  {
    String[] projection = { MediaStore.Images.Media.DATA } ;
    Cursor cursor =
    	managedQuery
    		(uri, projection, /* Which columns to return. */
    		 null, /* Which rows to return (all rows). */
    		 null, /* Selection arguments (none). */
    		 null  /* Order-by clause (ascending by name).*/
    		) ;
    int column_index =
      cursor.getColumnIndexOrThrow (MediaStore.Images.Media.DATA) ;
    cursor.moveToFirst () ;
    return (cursor.getString (column_index)) ;
  }
  
  
  
  /* Rem: window_width and window_height serve to allocate the layout.
     Then they are reminded to allow the movement method to only move a
     piece if it remains in the bound of the view.
     Attention, since the layout is allocated at game startup, even if
     the view dimensions change due to rotation, we can't simply change
     window_width and window_height to the new dimensions otherwise the
     layout would get inconsistent compared to the window_width and
     window_height. */
  private void startNewGame (String image_file_name, int nb_horiz_pieces,
      int nb_vert_pieces, float scale_factor, int window_width,
      int window_height, boolean rotate) 
  {
    Game game =
      GameAlloc.newGame
      (image_file_name, window_width, window_height,
       nb_horiz_pieces, nb_vert_pieces, scale_factor, rotate) ;
    if (game != null) {
      AndroJigApp.currentGame = game ;
      PlayAreaView.playAreaWidth = window_width ;
      PlayAreaView.playAreaHeight = window_height ;
      GameManips.shuffleAllPieces (game) ;
    }
    else this.showDialog (DIALOG_ID_ERROR_CREATING_GAME) ;
  }
}
