package francois.pessaux.androjig.android;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import francois.pessaux.android.R;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Environment;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.TextView.OnEditorActionListener;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Gallery;
import android.widget.ImageView;
import android.widget.TextView;


public class WebImagePickActivity extends Activity {
	private EditText editTextKeywords ;
	private Gallery gallerySearchResults ;
	private Button buttonOkWebImagePick ;
	private TextView textViewSelImgSize ;
	private Button buttonSearch ;
	
	
	/** String containing the keyword on which to perform the search in the
	    thread. In effect, the thread not being the one who created the view is
	    not allowed to access its components. So we pass it the keyword via this
	    global variable. */
	private String keywordForThread = null ;
	/** Integer telling at which page to start getting the search results among
	    the pages the search leads to. This global variable is there to allow
	    the search thread to get this value. It is incremented when we are
	    requested "Next Images", decremented when we are requested
	    "Previous Images" and reset when we start a new search by clicking on the
	    "Search" button. */
	private int startAtPageForThread = 0 ;
	private JSONObject sendSearchRequestResult ;
	/** Global variable set in threads to notify that an error occured and must
	    be reported via a dialog. The error code being 0 means no error, other
	    it is set to the id of the dialog that must be displayed to explain the
	    error. */
	private int duringThreadErrorCode ;
	/** Global variable used to make available for the [Gallery] Adapter the
	    images descriptions fetched and parsed for the search. */
	private SearchResult[] foundImagesDescriptions = null ;
	/** The Adapter used to handle and notify the [Gallery] that data (images
	    descriptions) changed. */
	private ImageAdapter galleryImageAdapter ;
	/** Records the currently selected image description index relative to the
	    [foundImagesDescriptions] array. If nothing is selected, this must be
	    -1. This selection is reset at each new search and each "Next" /
	    "Previous" image request. In other words, each time a selected image
	    is no more "visible", the selection disappears. */
	private int selectedImageIndexInDescriptions = -1 ;
	
	
	private static final int DIALOG_ID_ERROR_EMPTY_KEYWORD = 1 ;
	private static final int DIALOG_ID_ERROR_MALFORMED_URL = 2 ;
	private static final int DIALOG_ID_ERROR_MALFORMED_JSON = 3 ;
	private static final int DIALOG_ID_ERROR_NO_RESULT = 4 ;
	private static final int DIALOG_ID_ERROR_IMAGE_NOT_AVAILABLE = 5 ;
	
	
	/** Structure grouping the information describing each image found by the
	    search. */
	private class SearchResult {
		public int imageWidth ;
		public int imageHeight ;
	  public String imageUrl ;
	  public String thumbUrl ;
	  public ImageView thumbImageView ;
	}
	
	
	
	public class ImageAdapter extends BaseAdapter {
				
		/** Returns the number of images. */
		@Override
    public int getCount ()
    {
	    if (foundImagesDescriptions != null)
	    	return (foundImagesDescriptions.length) ;
	    else return (0) ;
    }

		
		/** Returns was is considered to be the data "bound" to the item at the
		    specified position. */
		@Override
    public Object getItem (int position)
    {
	    return (foundImagesDescriptions[position].imageUrl) ;
    }

		
		/** Returns the ID of an item. */
		@Override
    public long getItemId (int position)
    {
	    return (position) ;
    }

		
		/** Get a View that displays the data at the specified position in the data
		    set. */
		@Override
    public View getView (int position, View convertView, ViewGroup parent)
    {
			return (foundImagesDescriptions[position].thumbImageView) ;
    }
		
	}
	
	
	
	@Override
  public void onCreate (Bundle savedInstanceState)
  {
  	super.onCreate (savedInstanceState) ;
  	setContentView (R.layout.webimagepick) ;
  	buttonSearch = (Button) findViewById (R.id.buttonSearch) ;
  	editTextKeywords = (EditText) findViewById (R.id.editTextKeywords) ;
  	editTextKeywords.setOnEditorActionListener (editTextKeywordsInputListener) ;
  	gallerySearchResults = (Gallery) findViewById (R.id.gallerySearchResults) ;
  	buttonOkWebImagePick = (Button) findViewById (R.id.buttonOkWebImagePick) ;
  	textViewSelImgSize = (TextView) findViewById (R.id.textViewSelImgSize) ;
  	galleryImageAdapter = new ImageAdapter () ;
  	gallerySearchResults.setAdapter (galleryImageAdapter) ;
  	gallerySearchResults.setOnItemClickListener
  		(new OnItemClickListener () {
  			/** Callback invoked when an image is selected in the gallery. It
  			    reminds the selected image index in the array of images
  			    descriptions, update the TextView giving the image size and enables
  			    the "Ok" button. */
  			public void onItemClick (AdapterView<?> parent, View v, int position,
  															 long id) 
        {
  				selectedImageIndexInDescriptions = position ;
  				SearchResult descr = foundImagesDescriptions[position] ;
  				textViewSelImgSize.setText
  					(descr.imageWidth + "x" + descr.imageHeight) ;
  				buttonOkWebImagePick.setEnabled (true) ;
        }});
  	
  	foundImagesDescriptions = null ;
  	selectedImageIndexInDescriptions = -1 ;
  	keywordForThread = null ;
  	startAtPageForThread = 0 ;
  }
	
	
	
  @Override
  protected Dialog onCreateDialog (int id)
  {  	
    AlertDialog.Builder builder ;
    AlertDialog alert ;
    builder = new AlertDialog.Builder (this) ;
    switch (id) {
    case DIALOG_ID_ERROR_EMPTY_KEYWORD:
      builder.setMessage
      	(R.string.noKeywordStr)
        .setCancelable (false)
        .setPositiveButton
          (R.string.dismissStr, new DialogInterface.OnClickListener () {
          	public void onClick (DialogInterface dialog, int id) { }}) ;
      break ;
    	
    case DIALOG_ID_ERROR_MALFORMED_URL:
      builder.setMessage
				(R.string.malformedRequestStr)
				.setCancelable (false)
				.setPositiveButton
    			(R.string.dismissStr, new DialogInterface.OnClickListener () {
    				public void onClick (DialogInterface dialog, int id) { }}) ;
      break ;

    case DIALOG_ID_ERROR_MALFORMED_JSON:
      builder.setMessage
				(R.string.malformedResultStr)
				.setCancelable (false)
				.setPositiveButton
    			(R.string.dismissStr, new DialogInterface.OnClickListener () {
    				public void onClick (DialogInterface dialog, int id) { }}) ;
      break ;

    case DIALOG_ID_ERROR_NO_RESULT:
      builder.setMessage
  			(R.string.noResultStr)
  			.setCancelable (false)
  			.setPositiveButton
      		(R.string.dismissStr, new DialogInterface.OnClickListener () {
      			public void onClick (DialogInterface dialog, int id) { }}) ;
  	break ;
  	
    case DIALOG_ID_ERROR_IMAGE_NOT_AVAILABLE:
      builder.setMessage
				(R.string.imageNotAvailableStr)
				.setCancelable (false)
				.setPositiveButton
    			(R.string.dismissStr, new DialogInterface.OnClickListener () {
    				public void onClick (DialogInterface dialog, int id) { }}) ;
    	break ;
    	
    default:
      builder.setMessage (R.string.unknownMsgIdStr)
      .setCancelable (false)
      .setPositiveButton
      	(R.string.dismissStr, new DialogInterface.OnClickListener () {
      		public void onClick (DialogInterface dialog, int id) { }}) ;
      break ;
    }
    
    alert = builder.create () ;
    alert.show () ;
    return (super.onCreateDialog (id)) ;
  }
  
  
  /** Listener allowing to bind the "Enter" key of the keyword input widget in
      order to trigger a search, as if the "Search" button was pressed. */
  private OnEditorActionListener editTextKeywordsInputListener =
    new EditText.OnEditorActionListener () {
      public boolean onEditorAction (TextView v, int actionId, KeyEvent event)
      {
        if (actionId == EditorInfo.IME_ACTION_DONE) {
        	buttonSearch.performClick () ;
          return (true) ;
        }
        return (false) ;
      }} ;

      
	
	public void onClickCancelWebImagePick (View view)
	{
		setResult (RESULT_CANCELED, null) ;
		finish () ;
	}
	


	private class AsyncTaskOkWebImagePick extends AsyncTask<Void, Void, Void> {
		private ProgressDialog dialog ;
		private SearchResult imgInfo ;
		private File tmpFile ;
		
    @Override
    protected void onPreExecute ()
    {
    	dialog =
    		ProgressDialog.show
    			(WebImagePickActivity.this,
    			 getResources().getString (R.string.waitStr),
    			 getResources().getString (R.string.downloadingStr), true) ;
  		duringThreadErrorCode = 0 ;     /* Reset the error notification. */
  		imgInfo = foundImagesDescriptions[selectedImageIndexInDescriptions] ;
      File sdCard = Environment.getExternalStorageDirectory () ;
  		tmpFile = new File (sdCard, AndroJigApp.imgDownloadTmpFileName) ;
    }
    
		/** Fetch the image on the net and store it into a temporary file on the
		    device. */
		@Override
    protected Void doInBackground(Void... params)
    {
			HttpURLConnection conn = null ;
			InputStream in = null ;
			DataOutputStream out = null ;
			try {
				URL url = new URL (imgInfo.imageUrl) ;
				conn = (HttpURLConnection) url.openConnection () ;
				int length = conn.getContentLength () ;
				if (length <= 0) {
					/* No need to close connection, this will be done by the finally
					   clause that is always executed in the scope of the try
					   construct. */
					duringThreadErrorCode = DIALOG_ID_ERROR_IMAGE_NOT_AVAILABLE ;
					return (null) ;
				}
				
				byte imageBytes[] = new byte [length] ;
				in = conn.getInputStream () ;
				
				/* Read the bytes of the image. Since bytes can arrive by packets,
				   not all in one shot, we must loop until we read all the bytes. */
				boolean eofReached = false ;
				int storeAt = 0 ;
				int remLength = length ;
				while (! eofReached) {
						int reallyRead = in.read (imageBytes, storeAt, remLength) ;
						storeAt += reallyRead ;
						remLength -= reallyRead ;
						eofReached = (reallyRead == -1) ;
				}
				
				/* Now, write the content of the bytes buffer into a temporary
				   file. If the file already exists, then silently kill it. */
		    if (tmpFile.exists ()) tmpFile.delete () ;
		    /* Now, create the save game file. */
		    tmpFile.createNewFile () ;
		    out = new DataOutputStream (new FileOutputStream (tmpFile)) ;
		    out.write (imageBytes, 0, length) ;
			}
			catch (MalformedURLException e) {
				duringThreadErrorCode = DIALOG_ID_ERROR_MALFORMED_URL ;
			}
      catch (IOException e) {
      	duringThreadErrorCode = DIALOG_ID_ERROR_NO_RESULT ;
      }
      finally {
        /* Cleanup. Always done in the scope of the try. Even if a return
           instruction is encountered in the scope of the try. */
        try { if (in != null) in.close () ; } catch (IOException e) { }
        try { if (out != null) out.close () ; } catch (IOException e) { }
        if (conn != null) conn.disconnect () ;
      }
	    return (null) ;
    }

		
    @Override
    protected void onPostExecute (Void result)
    {
      dialog.dismiss () ;
  	  if (duringThreadErrorCode == 0) {
  	  	Intent data = new Intent () ;
  	  	data.setData (Uri.fromFile (tmpFile)) ;
  	  	setResult (RESULT_OK, data) ;
  	  	finish () ;
  	  }
  	  else showDialog (duringThreadErrorCode) ;
    }
	}
	
	
	
	public void onClickOkWebImagePick (View view)
	{
		new AsyncTaskOkWebImagePick ().execute () ;
	}

	
	
	public void onClickSearch (View view)
	{
    /* First, hide the keyboard. */
    InputMethodManager imm =
        (InputMethodManager) getSystemService (Context.INPUT_METHOD_SERVICE) ;
    imm.hideSoftInputFromWindow (editTextKeywords.getWindowToken (), 0) ;

		/* Because the thread we run to search is not the one who created the view
		  it is not allowed to access its components. So we pass it the keyword via
		  this global variable. */
		String keyword = editTextKeywords.getText ().toString() ;
		if (keyword == null) {
			showDialog (DIALOG_ID_ERROR_EMPTY_KEYWORD) ;
			return ;
		}
		
	  /* First, remove leading and trailing spaces. */
		keyword = keyword.trim () ;
		/* If the keyword is empty, directly return. */
		if (keyword.equals ("")) {
			showDialog (DIALOG_ID_ERROR_EMPTY_KEYWORD) ;
			return ;
		}
		
		/* Finally, set by side effect the keyword to let the search thread access
		   it. By the way, replace all spaces by a suitable characters sequence for
		   URLs. */
		keywordForThread = keyword.replaceAll (" ", "%20") ;
		startAtPageForThread = 0 ;
		new AsyncTaskDoSearch ().execute (null, null, null) ;
	}
	
	

	/** Assumes that [keywordForThread] and [startAtPageForThread] are properly
  		initialized. Invoked for:
   		 - a new search : [onClickSearch]
   		 - a search for previous images: [onClickPreviousImages]
   		 - a search for next images: [onClickNextImages].
  		!! USES-THREAD. */
	private class AsyncTaskDoSearch extends AsyncTask<Void, Void, Void> {
		private ProgressDialog dialog ;
		
    @Override
    protected void onPreExecute ()
    {
    	dialog =
    		ProgressDialog.show
    			(WebImagePickActivity.this,
    			 getResources().getString (R.string.waitStr),
    			 getResources().getString (R.string.searchingStr), true) ;
    	foundImagesDescriptions = null ;
    	/* Notify the adapter that data (images descriptions) changed. */
    	galleryImageAdapter.notifyDataSetChanged () ;
    	/* Disable Ok button since new search means that possible previous
    	   selected image is no more available hence relevant. */
    	buttonOkWebImagePick.setEnabled (false) ;
    	selectedImageIndexInDescriptions = -1 ;
    	textViewSelImgSize.setText ("-") ;
    }

		@Override
    protected Void doInBackground (Void... arg)
    {
			/* Set by side effect the result of the search. */
			sendSearchRequestResult =
				sendSearchRequest (keywordForThread, startAtPageForThread) ;
			return (null) ;
    }
			
    @Override
    protected void onPostExecute (Void result)
    {
      dialog.dismiss () ;
      /* Check if the search thread notified an error. */
      if (duringThreadErrorCode != 0) {
        showDialog (duringThreadErrorCode) ;
        return ;
      }
      
    	/* We now need to process the search result JSONObject set by side effect.
    	   in [sendSearchRequestResult]. */
    	if (sendSearchRequestResult == null) {
    		/* Should not arrive there because an error should have been notified and
    		   taken into account above. */
        showDialog (DIALOG_ID_ERROR_NO_RESULT) ;
        return ;
      }
    	/* Process the JSONObject result and set the result to be used by the
    	   [Gallery] adapter by side effect. */
    	foundImagesDescriptions = parseSearchResult (sendSearchRequestResult) ;
    	/* Notify the adapter that data (images descriptions) changed. */
    	galleryImageAdapter.notifyDataSetChanged () ;
    }
  }
		
	

	
	public void onClickClearSearckKeywords (View view)
	{
		editTextKeywords.setText ("") ;
	}
	
	
	
	public void onClickNextImages (View view)
	{
		/* Prevent "Next" if no search was initiated. */
		if ((foundImagesDescriptions == null) || (keywordForThread == null))
			return ;
		
		/* Currently, Google Search API only allows to get at most 64 pages of
		   results, each displaying 4 results. However, since pages refer to an
		   image, we are in fact limited to 64 images. */
		int nb_found = foundImagesDescriptions.length ;
		int tmp = Math.min (64, startAtPageForThread + nb_found) ;
		/* We only trigger HTTP request if we changed of page and if we remain in
		   a correct pages range. */
		if ((tmp <= 64) && (tmp != startAtPageForThread)) {
			startAtPageForThread = tmp ;
			new AsyncTaskDoSearch ().execute (null, null, null) ;
		}
	}


	
	public void onClickPreviousImages (View view)
	{
		/* Prevent "Previous" if no search was initiated. */
		if ((foundImagesDescriptions == null) || (keywordForThread == null))
			return ;

		int nb_found = foundImagesDescriptions.length ;
		int tmp = Math.max (0, startAtPageForThread - nb_found) ;
		/* We only trigger HTTP request if we changed of page and if we remain in
	   	 a correct pages range. */
		if ((tmp >= 0) && (tmp != startAtPageForThread)) {
			startAtPageForThread = tmp ;
			new AsyncTaskDoSearch ().execute (null, null, null) ;
		}
	}

	
	
	/** Temporary variable to let the thread that fetches thumbnails setting by
	    side effect the bitmaps it fetches.
	    !! USES-THREAD. */
	private Bitmap thumbBitmapForThread = null ;
	/** Function parsing the JSON object returned by the search and builds the
	    array containing the found images descriptions. */
	private SearchResult[] parseSearchResult (JSONObject json)
	{
		JSONObject jsonTmpData ;
		SearchResult[] imagesInfo ;
		/* Form of the results:
		  {
  	    "responseData" : {
          "results" : [],
          "cursor" : {}
   	    },
  	 	 "responseDetails" : null | string-on-error,
  	   "responseStatus" : 200 | error-code
			}
		 */
		try {
			jsonTmpData = json.getJSONObject ("responseData") ;
			/* We expect this object to have 2 fields: "results" (array) and
		   "cursor". We are especially interested in "results". */
			JSONArray results = jsonTmpData.getJSONArray ("results") ;
			/* Iterate on the results to build the set of found images information. */
			imagesInfo = new SearchResult[results.length ()] ;
			for (int i = 0; i < results.length (); i++) {
				final SearchResult imgInfo = new SearchResult () ;
				jsonTmpData = results.getJSONObject (i) ;
				imagesInfo[i] = imgInfo ;
				imgInfo.imageWidth = jsonTmpData.getInt ("width") ;
				imgInfo.imageHeight = jsonTmpData.getInt ("height") ;
				imgInfo.imageUrl = jsonTmpData.getString ("url") ;
				imgInfo.thumbUrl = jsonTmpData.getString ("tbUrl") ;
				
				Thread thread =
					new Thread () {
						/** Method called when the thread is launched via a call to its
						    method [start] from the class [Thread]. We need a thread because
						    performing HTTP requests is not allowed in the main application
						    thread. */
						@Override
						public void run ()
						{
							try {
								URL newurl = new URL (imgInfo.thumbUrl) ;
								thumbBitmapForThread =
									BitmapFactory.decodeStream
										(newurl.openConnection() .getInputStream ()) ;
							}
							/* TODO: INSTEAD, PUT SOMETHING TELLING THAT IMAGE IS NOT
							   AVAILABLE. */
							catch (MalformedURLException e) { thumbBitmapForThread = null ; }
			        catch (IOException e) { thumbBitmapForThread = null ; }
						}};
		  	thread.start () ;
		  	
		  	/* Now, wait for the thread to terminate. */
			  try { thread.join () ; }
			  /* TODO: INSTEAD, PUT SOMETHING TELLING THAT IMAGE IS NOT
			   	 AVAILABLE. */
			  catch (InterruptedException e) { thumbBitmapForThread = null ; }

			  ImageView thumbImageView = new ImageView (this) ;
				thumbImageView.setImageBitmap (thumbBitmapForThread) ;
				thumbImageView.setScaleType (ImageView.ScaleType.FIT_XY) ;
	      thumbImageView.setLayoutParams (new Gallery.LayoutParams (150, 120)) ;
	      imgInfo.thumbImageView = thumbImageView ;
			}    /* End of for (int i = 0; i < results.length (); i++). */
		}
    catch (JSONException e) {
    	return (null) ;  // TO ERROR MESSAGE.
    }
		
		/* Ok, now all images descriptions found are stored. */
		return (imagesInfo) ;
	}
	
	
	
	/** Send an HTTP request to Google Search Image with the given keyword.
	    !! CALLED-IN-THREAD. */
  private JSONObject sendSearchRequest (String keyword, int startAtPage)
	{
		URL url ;
		InputStream in = null ;
		HttpURLConnection urlConnection = null ;
		duringThreadErrorCode = 0 ;   /* Reset the error notification. */

		String urlStr =
			"https://ajax.googleapis.com/ajax/services/search/images?v=1.0&q=" +
			 keyword ;
		if (startAtPage > 0) urlStr = urlStr + "&start=" + startAtPage ;

		try {
			url = new URL (urlStr) ;

			urlConnection = (HttpURLConnection) url.openConnection () ;
			in = new BufferedInputStream (urlConnection.getInputStream ()) ;

			BufferedReader reader = new BufferedReader (new InputStreamReader (in)) ;
			StringBuilder builder = new StringBuilder () ;
			String line ;

			while ((line = reader.readLine ()) != null) builder.append (line) ;
	    
			/* Create the JSON object representing the result of the search. */
	    JSONObject json = new JSONObject (builder.toString ()) ;
	    /* If we arrive there, then the error code has remained set to 0 as it
	       was initialized at the beginning of the present function. */
	    return (json) ;
    }
    catch (MalformedURLException e) {
      duringThreadErrorCode = DIALOG_ID_ERROR_MALFORMED_URL ;
	    return (null) ;
    }
    catch (JSONException e) {
    	duringThreadErrorCode = DIALOG_ID_ERROR_MALFORMED_JSON ;
	    return (null) ;
    }
    catch (IOException e) {
      duringThreadErrorCode = DIALOG_ID_ERROR_NO_RESULT ;
	    return (null) ;
    }
    finally {
      /* Cleanup. */
      try { if (in != null) in.close () ; } catch (IOException e) { }
      if (urlConnection != null) urlConnection.disconnect () ;
      /* Especially no need of return statement, otherwise it would kill the
         other return statements since this block is always called !!! */
    }
	}
	
}
